use serde::ser::SerializeMap;
use serde::{Deserialize, Serialize, Serializer};

pub struct Command {
    command: Vec<String>,
    request_id: Option<u64>,
    asynchronous: bool,
}

impl Command {
    pub fn new(command: Vec<String>, request_id: Option<u64>) -> Self {
        Command {
            command,
            request_id,
            asynchronous: false,
        }
    }

    pub fn get_request_id(&self) -> &Option<u64> {
        return &self.request_id;
    }
}

impl Serialize for Command {
    fn serialize<S>(&self, serializer: S) -> Result<S::Ok, S::Error>
    where
        S: Serializer,
    {
        let mut map = serializer.serialize_map(None)?;
        map.serialize_entry("command", &self.command)?;
        self.request_id
            .map(|reqid| map.serialize_entry("request_id", &reqid))
            .map_or(Ok(None), |x| x.map(Some))?;
        if self.asynchronous {
            map.serialize_entry("async", &self.asynchronous)?;
        }
        map.end()
    }
}
